/*********************************************************************/
/*                                                                   */
/*             Optimized BLAS libraries                              */
/*                     By Kazushige Goto <kgoto@tacc.utexas.edu>     */
/*                                                                   */
/* Copyright (c) The University of Texas, 2005. All rights reserved. */
/* UNIVERSITY EXPRESSLY DISCLAIMS ANY AND ALL WARRANTIES CONCERNING  */
/* THIS SOFTWARE AND DOCUMENTATION, INCLUDING ANY WARRANTIES OF      */
/* MERCHANTABILITY, FITNESS FOR ANY PARTICULAR PURPOSE,              */
/* NON-INFRINGEMENT AND WARRANTIES OF PERFORMANCE, AND ANY WARRANTY  */
/* THAT MIGHT OTHERWISE ARISE FROM COURSE OF DEALING OR USAGE OF     */
/* TRADE. NO WARRANTY IS EITHER EXPRESS OR IMPLIED WITH RESPECT TO   */
/* THE USE OF THE SOFTWARE OR DOCUMENTATION.                         */
/* Under no circumstances shall University be liable for incidental, */
/* special, indirect, direct or consequential damages or loss of     */
/* profits, interruption of business, or related expenses which may  */
/* arise from use of Software or Documentation, including but not    */
/* limited to those resulting from defects in Software and/or        */
/* Documentation, or loss or inaccuracy of data of any kind.         */
/*********************************************************************/

#include "common.h"
double fabs(double);

#ifdef DOUBLE
#define RETURNTYPE double
#else
#define RETURNTYPE FLOATRET
#endif

#if defined(F_INTERFACE) && defined(XDOUBLE)
void NAME(FLOAT *result, blasint *N, FLOAT *dx, blasint *INCX){

  long n = *N;
  long incx = *INCX;
#elif defined F_INTERFACE
RETURNTYPE NAME(blasint *N, FLOAT *dx, blasint *INCX){

  long n = *N;
  long incx = *INCX;
#else
FLOAT CNAME(long n, FLOAT *dx, long incx){
#endif

  long i;

  FLOAT d1, dmax;

  if (n < 1 || incx <= 0) {
#if defined(F_INTERFACE) && defined(XDOUBLE)
    *result = ZERO;
    return;
#else
    return 0.;
#endif
  }


  dmax = fabs(*dx);
  dx += incx;

  if (n == 1) {
#if defined(F_INTERFACE) && defined(XDOUBLE)
    *result = dmax;
    return;
#else
    return dmax;
#endif
  }

  i = n - 1;

  do {
    d1 = fabs(*dx);
    dx += incx;
    if (d1 > dmax) dmax = d1;
    i --;
  } while (i > 0);

#if defined(F_INTERFACE) && defined(XDOUBLE)
  *result = dmax;
  return;
#else
 return dmax;
#endif

}
